@@ -1,10 +1,7 @@
 import os
 import sys
-sys.path.insert(
-    0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
 import cv2
 import numpy as np
-import six
 import glob
 from PIL import Image
 import pickle
@@ -35,7 +32,16 @@ def load_pickle_file(fname):
         return None
 
 
-def resize_image(img: np.ndarray, target_size: int, interpolation=None) -> np.ndarray:
+def read_image_to_np(img_path: str) -> np.ndarray:
+    img = cv2.imread(img_path, cv2.IMREAD_COLOR)
+    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)  # convert BGR to RGB.
+    # h, w, c = img.shape
+    # assert c in [1, 3]
+    # assert c.dytpe == np.unit8
+    return img
+
+
+def resize_image(img: np.ndarray, target_size: int, interpolation=cv2.INTER_LINEAR) -> np.ndarray:
     """resize image with shorter edge equal to target_size.
 
     Args:
@@ -46,14 +52,16 @@ def resize_image(img: np.ndarray, target_size: int, interpolation=None) -> np.nd
     Returns:
         resized image data
     """
-    percent = float(target_size) / min(img.shape[0], img.shape[1])
-    resized_width = int(round(img.shape[1] * percent))
-    resized_height = int(round(img.shape[0] * percent))
-    if interpolation:
-        resized = cv2.resize(
-            img, (resized_width, resized_height), interpolation=interpolation)
-    else:
-        resized = cv2.resize(img, (resized_width, resized_height))
+
+    h, w, c = img.shape
+    assert c in [1, 3]
+
+    percent = float(target_size) / min(h, w)
+    resized_width = int(round(w * percent))
+    resized_height = int(round(h * percent))
+
+    resized = cv2.resize(img, (resized_width, resized_height), interpolation=interpolation)
+    # assert resized.dytpe == np.float32
     return resized
 
 
@@ -86,13 +94,13 @@ def preprocess_image(img: np.ndarray, random_mirror=False) -> np.ndarray:
     """
     image(uint8) to tensor(float32). scaled by 1/255, centered, standarized.
     :param img: np.ndarray: shape: [ns, h, w, 3], color order: rgb.
-    :return: np.ndarray: shape: [ns, c, h, w]
+    :return: np.ndarray: shape: [ns, 3, h, w]
     """
     # ImageNet stats.
     mean = [0.485, 0.456, 0.406]
     std = [0.229, 0.224, 0.225]
 
-    # transpose to [ns, c, h, w]
+    # transpose from [ns, h, w, 3] to [ns, c, h, w], and scaled by 1/255
     img = img.astype('float32').transpose((0, 3, 1, 2)) / 255
 
     img_mean = np.array(mean).reshape((3, 1, 1))
@@ -110,9 +118,9 @@ def preprocess_image(img: np.ndarray, random_mirror=False) -> np.ndarray:
 
 def read_image(img_path, target_size=256, crop_size=224, crop=True) -> np.ndarray:
     """
-    resize_short to 256, then center crop to 224.
+    resize_short to target_size, then center crop to crop_size or not crop.
     :param img_path: one image path
-    :return: np.ndarray: shape: [1, h, w, 3], color order: rgb.
+    :return: np.ndarray: shape: [1, h, w, 3], dtype: uint8, color order: rgb.
     """
 
     if isinstance(img_path, str):
@@ -120,10 +128,9 @@ def read_image(img_path, target_size=256, crop_size=224, crop=True) -> np.ndarra
             img = Image.open(f)
             img = img.convert('RGB')
             img = np.array(img)
-            img = resize_image(img, target_size, interpolation=None)
+            img = resize_image(img, target_size)
             if crop:
                 img = crop_image(img, target_size=crop_size, center=True)
-            # img = img[:, :, ::-1]
             img = np.expand_dims(img, axis=0)
             return img
     elif isinstance(img_path, np.ndarray):
@@ -133,16 +140,16 @@ def read_image(img_path, target_size=256, crop_size=224, crop=True) -> np.ndarra
         ValueError(f"Not recognized data type {type(img_path)}.")
 
 
-def restore_image(img: np.ndarray) -> np.ndarray:
+def restore_image(float_input_data: np.ndarray) -> np.ndarray:
     mean = [0.485, 0.456, 0.406]
     std = [0.229, 0.224, 0.225]
     img_mean = np.array(mean).reshape((3, 1, 1))
     img_std = np.array(std).reshape((3, 1, 1))
-    img *= img_std
-    img += img_mean
-    img *= 255
-    img += 0.5  # for float to integer
-    img = np.uint8(img.transpose((0, 2, 3, 1)))
+    float_input_data *= img_std
+    float_input_data += img_mean
+    float_input_data *= 255
+    float_input_data += 0.5  # for float to integer
+    img = np.uint8(float_input_data.transpose((0, 2, 3, 1)))
     return img
 
 
@@ -224,11 +231,58 @@ def extract_img_paths(directory):
     return img_paths, img_names
 
 
+
+def images_transform_pipeline(array_or_path, resize_to=224, crop_to=None):
+    """[summary]
+
+    Args:
+        array_or_path ([type]): [description]
+        resize_to (int, optional): [description]. Defaults to 224.
+        crop_to ([type], optional): [description]. Defaults to None.
+    """
+    if crop_to is not None:
+        assert isinstance(crop_to, int)
+        def read_image_func(path):
+            return read_image(path, target_size=resize_to, crop_size=crop_to)
+    else:
+        def read_image_func(path):
+            return read_image(path, target_size=resize_to, crop=False)
+
+    if isinstance(array_or_path, str):
+        # one single image path.
+        uint8_imgs = read_image_func(array_or_path)
+        float_input_data = preprocess_image(uint8_imgs)
+    elif isinstance(array_or_path, list) and all(
+            isinstance(elem, str) for elem in array_or_path):
+        # a list of image paths.
+        uint8_imgs = []
+        for fp in array_or_path:
+            uint8_img = read_image_func(fp)
+            uint8_imgs.append(uint8_img)
+        uint8_imgs = np.concatenate(uint8_imgs)
+        float_input_data = preprocess_image(uint8_imgs)
+    else:
+        # an array.
+        if len(array_or_path.shape) == 3:
+            uint8_imgs = np.expand_dims(array_or_path, axis=0)
+
+        if np.issubdtype(array_or_path.dtype, np.integer):
+            # array_or_path is an image.
+            uint8_imgs = uint8_imgs.copy()
+            float_input_data = preprocess_image(uint8_imgs)
+        else:
+            # array_or_path is float input data.
+            uint8_imgs = restore_image(array_or_path.copy())
+            float_input_data = array_or_path
+
+    return uint8_imgs, float_input_data
+
+
 def preprocess_inputs(inputs, model_input_shape):
     """[summary]
 
     Args:
-        inputs ([type]): [description]
+        inputs ([type]): can be a str, a list of str, or np.ndarray.
         model_input_shape ([type]): [description]
 
     Returns:
@@ -238,7 +292,7 @@ def preprocess_inputs(inputs, model_input_shape):
     if isinstance(inputs, str):
         imgs = read_image(inputs, crop_size=model_input_shape[1])
         data = preprocess_image(imgs)
-    elif bool(list) and isinstance(inputs, list) and all(
+    elif isinstance(inputs, list) and all(
             isinstance(elem, str) for elem in inputs):
         imgs = []
         for fp in inputs:
