@@ -16,7 +16,7 @@ class Interpreter(ABC):
 
     """
 
-    def __init__(self, paddle_model, device, use_cuda, **kwargs):
+    def __init__(self, paddle_model: callable, device: str, use_cuda: bool, **kwargs):
         """
 
         Args:
@@ -39,8 +39,9 @@ class Interpreter(ABC):
 
         self.device = device
         self.paddle_model = paddle_model
+        self.predict_fn = None
 
-    def _paddle_prepare(self, predict_fn=None):
+    def _paddle_prepare(self, predict_fn: callable or None=None):
         """
         Prepare Paddle program inside of the interpreter. This will be called by interpret().
         Would be renamed to ``_build_predict_fn``.
@@ -72,16 +73,18 @@ class Interpreter(ABC):
 class InputGradientInterpreter(Interpreter):
     """Input Gradient based Interpreter.
 
+    Interpreters that are derived from InputGradientInterpreter:
+    GradShapCVInterpreter, IntGradCVInterpreter, SmoothGradInterpreter
+
     """
 
-    def __init__(self, paddle_model, device, use_cuda, **kwargs):
+    def __init__(self, paddle_model: callable, device: str, use_cuda: bool, **kwargs):
         Interpreter.__init__(self, paddle_model, device, use_cuda, **kwargs)
         assert hasattr(paddle_model, 'forward') and hasattr(paddle_model, 'backward'), \
             "paddle_model has to be " \
             "an instance of paddle.nn.Layer or a compatible one."
-        self.predict_fn = None
 
-    def _build_predict_fn(self, rebuild=False, gradient_of='probability'):
+    def _build_predict_fn(self, rebuild: bool=False, gradient_of: str='probability'):
         """Build ``self.predict_fn`` for input gradients based algorithms.
         The model is supposed to be a classification model.
 
@@ -209,16 +212,18 @@ class InputGradientInterpreter(Interpreter):
 class InputOutputInterpreter(Interpreter):
     """Input-Output Correlation based Interpreter.
 
+    Interpreters that are derived from InputOutputInterpreter:
+    OcclusionInterpreter, LIMECVInterpreter
+
     """
 
-    def __init__(self, paddle_model, device, use_cuda, **kwargs):
+    def __init__(self, paddle_model: callable, device: str, use_cuda: bool, **kwargs):
         Interpreter.__init__(self, paddle_model, device, use_cuda, **kwargs)
         assert hasattr(paddle_model, 'forward'), \
             "paddle_model has to be " \
             "an instance of paddle.nn.Layer or a compatible one."
-        self.predict_fn = None
 
-    def _build_predict_fn(self, rebuild=False, output='probability'):
+    def _build_predict_fn(self, rebuild: bool=False, output: str='probability'):
         """Build self.predict_fn for Input-Output based algorithms.
         The model is supposed to be a classification model.
 
@@ -296,16 +301,18 @@ class IntermediateLayerInterpreter(Interpreter):
     """Interpreter that exhibits features from intermediate layers to produce explanations.
     This interpreter extracts one layer's feature.
 
+    Interpreters that are derived from IntermediateLayerInterpreter:
+    ScoreCAMInterpreter
+
     """
 
-    def __init__(self, paddle_model, device, use_cuda, **kwargs):
+    def __init__(self, paddle_model: callable, device: str, use_cuda: bool, **kwargs):
         Interpreter.__init__(self, paddle_model, device, use_cuda, **kwargs)
         assert hasattr(paddle_model, 'forward'), \
             "paddle_model has to be " \
             "an instance of paddle.nn.Layer or a compatible one."
-        self.predict_fn = None
 
-    def _build_predict_fn(self, rebuild=False, target_layer=None):
+    def _build_predict_fn(self, rebuild: bool=False, target_layer: str=None):
         """Build self.predict_fn for IntermediateLayer based algorithms.
         The model is supposed to be a classification model.
         An example of ``predict_fn`` is
@@ -324,7 +331,7 @@ class IntermediateLayerInterpreter(Interpreter):
                         h = v.register_forward_post_hook(hook)
                         hooks.append(h)
 
-                assert len(hooks) == 1, f"target_layer `{target_layer}`` does not exist in the given model, \
+                assert len(hooks) == 1, f"target_layer `{target_layer}`` does not exist in the given model, \n \
                                 the list of layer names are \n \
                                 {[n for n, v in self.paddle_model.named_sublayers()]}"
                 
