@@ -1,8 +1,7 @@
 import numpy as np
-import paddle
 from tqdm import tqdm
 from .abc_interpreter import InputGradientInterpreter, Interpreter
-from ..data_processor.readers import preprocess_inputs, preprocess_save_path
+from ..data_processor.readers import images_transform_pipeline, preprocess_save_path
 from ..data_processor.visualizer import explanation_to_vis, show_vis_explanation, save_image
 
 
@@ -14,31 +13,33 @@ class IntGradCVInterpreter(InputGradientInterpreter):
     https://arxiv.org/abs/1703.01365
     """
 
-    def __init__(self,
-                 paddle_model,
-                 use_cuda=None,
-                 device='gpu:0',
-                 model_input_shape=[3, 224, 224]) -> None:
+    def __init__(
+        self,
+        paddle_model: callable,
+        use_cuda: bool=True,
+        device: str='gpu:0'
+    ):
         """
 
         Args:
             paddle_model (callable): A model with ``forward`` and possibly ``backward`` functions.
             device (str): The device used for running `paddle_model`, options: ``cpu``, ``gpu:0``, ``gpu:1`` etc.
             use_cuda (bool):  Would be deprecated soon. Use ``device`` directly.
-            model_input_shape (list, optional): The input shape of the model. Default: [3, 224, 224]
         """
         InputGradientInterpreter.__init__(self, paddle_model, device, use_cuda)
 
-        self.model_input_shape = model_input_shape
-
-    def interpret(self,
-                  inputs,
-                  labels=None,
-                  baselines=None,
-                  steps=50,
-                  num_random_trials=10,
-                  visual=True,
-                  save_path=None):
+    def interpret(
+        self,
+        inputs: str or list(str) or np.ndarray,
+        labels: list or tuple or np.ndarray or None=None,
+        baselines: np.ndarray or None=None,
+        steps: int=50,
+        num_random_trials: int=10,
+        resize_to=224, 
+        crop_to=None,
+        visual: bool=True,
+        save_path=None
+    ):
         """Main function of the interpreter.
 
         Args:
@@ -48,6 +49,9 @@ class IntGradCVInterpreter(InputGradientInterpreter):
                                                         If None, the baselines of all zeros will be used. Default: None.
             steps (int, optional): number of steps in the Riemman approximation of the integral. Default: 50
             num_random_trials (int, optional): number of random initializations to take average in the end. Default: 10
+            resize_to (int, optional): [description]. Images will be rescaled with the shorter edge being `resize_to`. Defaults to 224.
+            crop_to ([type], optional): [description]. After resize, images will be center cropped to a square image with the size `crop_to`. 
+                If None, no crop will be performed. Defaults to None.
             visual (bool, optional): Whether or not to visualize the processed image. Default: True
             save_path (str or list of strs or None, optional): The filepath(s) to save the processed image(s). If None, the image will not be saved. Default: None
 
@@ -55,7 +59,7 @@ class IntGradCVInterpreter(InputGradientInterpreter):
             [numpy.ndarray]: interpretations/gradients for images
         """
 
-        imgs, data = preprocess_inputs(inputs, self.model_input_shape)
+        imgs, data = images_transform_pipeline(inputs, resize_to, crop_to)
 
         self.data_type = np.array(data).dtype
         self.input_type = type(data)
@@ -126,108 +130,146 @@ class IntGradNLPInterpreter(Interpreter):
 
         Args:
             paddle_model (callable): A model with ``forward`` and possibly ``backward`` functions.
-            use_cuda (bool):  Would be deprecated soon. Use ``device`` directly.
             device (str): The device used for running `paddle_model`, options: ``cpu``, ``gpu:0``, ``gpu:1`` etc.
-
+            use_cuda (bool):  Would be deprecated soon. Use ``device`` directly.
         """
         Interpreter.__init__(self, paddle_model, device, use_cuda)
-        self.paddle_model = paddle_model
-        self.use_cuda = use_cuda
-        self.paddle_prepared = False
 
     def interpret(self,
                   data,
                   labels=None,
                   steps=50,
-                  return_pred=True,
-                  visual=True):
-        """
-        Main function of the interpreter.
+                  embedding_name='word_embeddings',
+                  return_pred=True):
+        """Main function of the interpreter.
 
         Args:
-            data (tuple or paddle.tensor): The inputs to the NLP model.
-            label (list or numpy.ndarray, optional): The target label to analyze. If None, the most likely label will be used. Default: None.
-            noise_amount (float, optional): Noise level of added noise to the embeddings.
-                                            The std of Guassian random noise is noise_amount * (x_max - x_min). Default: 0.1
-            return_pred (bool, optional): Whether or not to return predicted labels and probabilities. If True, a tuple of predicted labels, probabilities, and interpretations will be returned.
-                                        There are useful for visualization. Else, only interpretations will be returned. Default: False.
-            visual (bool, optional): Whether or not to visualize. Default: True.
-        
+            data ([type]): [description]
+            labels ([type], optional): The target label to analyze. If None, the most likely label will be used. Default: None.
+            steps (int, optional): number of steps in the Riemman approximation of the integral. Default: 50
+            embedding_name (str, optional): name of the embedding layer at which the steps will be applied. 
+                Defaults to 'word_embeddings'. The correct name of embedding can be found through ``print(model)``.
+            return_pred (bool, optional): Whether or not to return predicted labels and probabilities. 
+                If True, a tuple of predicted labels, probabilities, and interpretations will be returned.
+                There are useful for visualization. Else, only interpretations will be returned. Default: True.
+
         Returns:
             [numpy.ndarray or tuple]: interpretations for each word or a tuple of predicted labels, probabilities, and interpretations.
         """
 
-        if not self.paddle_prepared:
-            self._paddle_prepare()
+        self._build_predict_fn(embedding_name=embedding_name, gradient_of='probability')
 
         if isinstance(data, tuple):
-            n = data[0].shape[0]
+            bs = data[0].shape[0]
         else:
-            n = data.shape[0]
+            bs = data.shape[0]
 
-        self._alpha = 1
-        gradients, out, data_out = self.predict_fn(data, [0] * n)
-        out = paddle.nn.functional.softmax(paddle.to_tensor(out)).numpy()
-        
-        if labels is None:
-            labels = np.argmax(out, axis=1)
-        else:
-            labels = np.array(labels)
-
-        labels = labels.reshape((n, ))
+        gradients, labels, data_out, probas = self.predict_fn(data, labels, None)
+        labels = labels.reshape((bs, ))
         total_gradients = np.zeros_like(gradients)
         for alpha in np.linspace(0, 1, steps):
-            self._alpha = alpha
-            gradients, _, emb = self.predict_fn(data, labels)
+            gradients, _, _, _ = self.predict_fn(data, labels, alpha)
             total_gradients += gradients
-
+        
         ig_gradients = total_gradients * data_out / steps
-
-        #avg_gradients = np.average(np.array(gradients_list), axis=0)
+        ig_gradients = np.sum(ig_gradients, axis=-1)
 
         if return_pred:
-            out = np.array(out)
-            return labels, [o[labels[i]]
-                            for i, o in enumerate(out)], ig_gradients
+            return labels, probas.numpy(), ig_gradients
+        
+        # Visualization is currently not supported here.
+        # See the tutorial for more information:
+        # https://github.com/PaddlePaddle/InterpretDL/blob/master/tutorials/ernie-2.0-en-sst-2-tutorials.ipynb
 
         return ig_gradients
 
-    def _paddle_prepare(self, predict_fn=None):
-        if predict_fn is None:
+    def _build_predict_fn(self, rebuild=False, embedding_name='word_embeddings', gradient_of='probability'):
+        
+        if self.predict_fn is not None:
+            assert callable(self.predict_fn), "predict_fn is predefined before, but is not callable." \
+                "Check it again."
+            return
+        
+        import paddle
+        if self.predict_fn is None or rebuild:
+            assert gradient_of in ['loss', 'logit', 'probability']
+
+            if not paddle.is_compiled_with_cuda() and self.device[:3] == 'gpu':
+                print("Paddle is not installed with GPU support. Change to CPU version now.")
+                self.device = 'cpu'
+
+            # set device. self.device is one of ['cpu', 'gpu:0', 'gpu:1', ...]
             paddle.set_device(self.device)
 
+            # to get gradients, the ``train`` mode must be set.
             self.paddle_model.train()
-
-            self._embedding = None
-
-            def hook(layer, input, output):
-                output = self._alpha * output
-                self._embedding = output
-                return output
-
+            
+            # later version will be simplied.
             for n, v in self.paddle_model.named_sublayers():
-                if "embedding" in v.__class__.__name__.lower():
-                    v.register_forward_post_hook(hook)
                 if "batchnorm" in v.__class__.__name__.lower():
                     v._use_global_stats = True
                 if "dropout" in v.__class__.__name__.lower():
                     v.p = 0
-
-            def predict_fn(data, labels):
-                global embedding
+                    
+            def predict_fn(data, labels, noise_scale=1.0):
                 if isinstance(data, tuple):
-                    probs = self.paddle_model(*data)
+                    # NLP models usually have two inputs.
+                    bs = data[0].shape[0]
+                    data = (paddle.to_tensor(data[0]), paddle.to_tensor(data[1]))
                 else:
-                    probs = self.paddle_model(data)
-                labels_onehot = paddle.nn.functional.one_hot(
-                    paddle.to_tensor(labels), num_classes=probs.shape[1])
-                target = paddle.sum(probs * labels_onehot, axis=1)
-                target.backward()
-                gradients = self._embedding.grad
+                    bs = data.shape[0]
+                    data = paddle.to_tensor(data)
+                    
+                assert labels is None or \
+                    (isinstance(labels, (list, np.ndarray)) and len(labels) == bs)
+            
+                target_feature_map = []
+                def hook(layer, input, output):
+                    if noise_scale is not None:
+                        output = noise_scale * output
+                    target_feature_map.append(output)
+                    return output
+                hooks = []
+                for name, v in self.paddle_model.named_sublayers():
+                    if embedding_name in name:
+                        h = v.register_forward_post_hook(hook)
+                        hooks.append(h)
+                        
+                if isinstance(data, tuple):
+                    logits = self.paddle_model(*data)  # get logits, [bs, num_c]
+                else:
+                    logits = self.paddle_model(data)  # get logits, [bs, num_c]
+                    
+                for h in hooks:
+                    h.remove()
+                    
+                probas = paddle.nn.functional.softmax(logits, axis=1)  # get probabilities.
+                preds = paddle.argmax(probas, axis=1)  # get predictions.
+                if labels is None:
+                    labels = preds.numpy()  # label is an integer.
+                
+                if gradient_of == 'loss':
+                    # loss
+                    loss = paddle.nn.functional.cross_entropy(
+                        logits, paddle.to_tensor(labels), reduction='sum'
+                    )
+                else:
+                    # logits or probas
+                    labels = np.array(labels).reshape((bs, ))
+                    labels_onehot = paddle.nn.functional.one_hot(
+                        paddle.to_tensor(labels), num_classes=probas.shape[1]
+                    )
+                    if gradient_of == 'logit':
+                        loss = paddle.sum(logits * labels_onehot, axis=1)
+                    else:
+                        loss = paddle.sum(probas * labels_onehot, axis=1)
+
+                loss.backward()
+                gradients = target_feature_map[0].grad  # get gradients of "embedding".
+                loss.clear_gradient()
+
                 if isinstance(gradients, paddle.Tensor):
                     gradients = gradients.numpy()
-                return gradients, probs.numpy(), self._embedding.numpy(
-                )
+                return gradients, labels, target_feature_map[0].numpy(), probas
 
         self.predict_fn = predict_fn
-        self.paddle_prepared = True
