@@ -4,7 +4,7 @@ import paddle
 import re
 
 from .abc_interpreter import Interpreter
-from ..data_processor.readers import preprocess_inputs, preprocess_save_path
+from ..data_processor.readers import images_transform_pipeline, preprocess_save_path
 from ..data_processor.visualizer import explanation_to_vis, show_vis_explanation, save_image
 
 
@@ -19,18 +19,15 @@ class RolloutInterpreter(Interpreter):
     def __init__(self,
                  paddle_model,
                  use_cuda=None,
-                 device='gpu:0',
-                 model_input_shape=[3, 224, 224]) -> None:
+                 device='gpu:0') -> None:
         """
 
         Args:
             paddle_model (callable): A model with ``forward`` and possibly ``backward`` functions.
             device (str): The device used for running `paddle_model`, options: ``cpu``, ``gpu:0``, ``gpu:1`` etc.
             use_cuda (bool):  Would be deprecated soon. Use ``device`` directly.
-            model_input_shape (list, optional): The input shape of the model. Default: [3, 224, 224]
         """
         Interpreter.__init__(self, paddle_model, device, use_cuda)
-        self.model_input_shape = model_input_shape
         self.paddle_prepared = False
 
         # init for usages during the interpretation.
@@ -39,6 +36,8 @@ class RolloutInterpreter(Interpreter):
     def interpret(self,
                   inputs,
                   start_layer=0,
+                  resize_to=224, 
+                  crop_to=None,
                   visual=True,
                   save_path=None):
         """
@@ -46,7 +45,11 @@ class RolloutInterpreter(Interpreter):
 
         Args:
             inputs (str or list of strs or numpy.ndarray): The input image filepath or a list of filepaths or numpy array of read images.
-            labels (list or tuple or numpy.ndarray, optional): The target labels to analyze. The number of labels should be equal to the number of images. If None, the most likely label for each image will be used. Default: None
+            labels (list or tuple or numpy.ndarray, optional): The target labels to analyze. The number of labels should be equal to the number of images. 
+                If None, the most likely label for each image will be used. Default: None
+            resize_to (int, optional): [description]. Images will be rescaled with the shorter edge being `resize_to`. Defaults to 224.
+            crop_to ([type], optional): [description]. After resize, images will be center cropped to a square image with the size `crop_to`. 
+                If None, no crop will be performed. Defaults to None.
             visual (bool, optional): Whether or not to visualize the processed image. Default: True
             save_path (str or list of strs or None, optional): The filepath(s) to save the processed image(s). If None, the image will not be saved. Default: None
 
@@ -54,7 +57,7 @@ class RolloutInterpreter(Interpreter):
             [numpy.ndarray]: interpretations/heatmap for images
         """
 
-        imgs, data = preprocess_inputs(inputs, self.model_input_shape)
+        imgs, data = images_transform_pipeline(inputs, resize_to, crop_to)
         bsz = len(data)  # batch size
         save_path = preprocess_save_path(save_path, bsz)
 
